@@ -1,33 +1,17 @@
 #' Draw from the marginal posteriors of a tidylda topic model
-#' @description These functions are used to sample from the marginal posteriors
-#'   of a \code{tidylda} topic model. This is useful for quantifying uncertainty
-#'   around the parameters of \code{beta} or \code{theta}.
-#' @param x For \code{posterior}, an object of class
-#'   \code{tidylda}. For \code{generate}, an object of class
-#'   \code{tidylda_posterior} obtained by a call to \code{posterior}.
+#' @description Sample from the marginal posteriors of a \code{tidylda} topic
+#'   model. This is useful for quantifying uncertainty around the parameters of
+#'   \code{beta} or \code{theta}.
+#' @param x An object of class \code{tidylda}. 
 #' @param matrix A character of either 'theta' or 'beta', indicating from which
 #'   matrix to draw posterior samples.
 #' @param which Row index of \code{theta}, for document, or \code{beta}, for
 #'   topic, from which to draw samples. \code{which} may also be a vector of
-#'   indices.
-#' @param times Integer number of samples to draw.
+#'   indices to sample from multiple documents or topics simultaneously.
+#' @param times Integer, number of samples to draw.
 #' @param ... Other arguments, currently not used.
 #' @return 
-#' \code{posterior} returns an object of class \code{tidylda_posterior}.
-#' 
-#' \code{generate} returns a tibble with one row per parameter per sample.
-#' @details
-#' To sample from the marginal posteriors of a model, you must first make a call
-#' to \code{posterior} and then a call to \code{generate}.
-#' 
-#' \code{posterior} takes an object of class \code{tidylda} and constructs an
-#' object of class \code{tidylda_posterior} which contains two matrices. The rows
-#' of these matrices are Dirichlet parameters used to sample from the marginal
-#' posteriors of \code{theta} and \code{beta}.
-#' 
-#' \code{generate} takes an object of class \code{tidylda_posterior} and samples
-#' from the marginal posterior of the parameters specified by the \code{matrix}
-#' argument.
+#' \code{posterior} returns a tibble with one row per parameter per sample.
 #' @references 
 #' Heinrich, G. (2005) Parameter estimation for text analysis. Technical report. 
 #' \href{http://www.arbylon.net/publications/text-est.pdf}{http://www.arbylon.net/publications/text-est.pdf}
@@ -48,22 +32,19 @@
 #'   iterations = 200, burnin = 175
 #' )
 #' 
-#' # construct a posterior object
-#' p <- posterior(m)
-#' 
 #' # sample from the marginal posterior corresponding to topic 1
-#' t1 <- generate(
-#'   x = p,
+#' t1 <- posterior(
+#'   x = m,
 #'   matrix = "beta",
 #'   which = 1,
 #'   times = 100  
 #' )
 #' 
-#' # sample from the marginal posterior corresponding to document 5
-#' d5 <- generate(
-#'   x = p,
+#' # sample from the marginal posterior corresponding to documents 5 and 6
+#' d5 <- posterior(
+#'   x = m,
 #'   matrix = "theta",
-#'   which = 5,
+#'   which = c(5, 6),
 #'   times = 100
 #' )
 #' }
@@ -73,50 +54,8 @@ posterior <- function(x, ...) UseMethod("posterior")
 #' Posterior method for tidylda
 #' @rdname posterior
 #' @export
-posterior.tidylda <- function(x, ...) {
-  
-  # get proper alpha
-  alpha <- format_alpha(
-    x$alpha, 
-    k = nrow(x$beta)
-  )
-  
-  # get proper eta
-  eta <- format_eta(
-    x$eta, 
-    k = nrow(x$beta), 
-    Nv = ncol(x$beta)
-  )
-  
-  # extract dirichlet parameters for theta
-  theta_par <- t(x$counts$Cd) + alpha$alpha
-  
-  colnames(theta_par) <- rownames(x$theta)
-  rownames(theta_par) <- colnames(x$theta)
-  
-  # extract dirichlet parameters for beta
-  beta_par <- x$counts$Cv + eta$eta
-  
-  rownames(beta_par) <- rownames(x$beta)
-  colnames(beta_par) <- colnames(x$beta)
-  
-  beta_par <- t(beta_par)
-  
-  # prepare and return result
-  result <- list(
-    theta_par = theta_par,
-    beta_par = beta_par
-  )
-  
-  class(result) <- "tidylda_posterior"
-  
-  result
-}
-
-#' @rdname posterior
-#' @export
-generate.tidylda_posterior <- function(
-  x,
+posterior.tidylda <- function(
+  x, 
   matrix,
   which,
   times,
@@ -148,43 +87,95 @@ generate.tidylda_posterior <- function(
     stop("times must be a positive number.")
   }
   
-  # extract the right x 
+  # construct Dirichlet parameter(s) based on "matrix" argument
   if (matrix[1] == "theta") {
-    obj <- x$theta_par
-    # do one more check on which
-    if (any(which > ncol(obj))) {
-      stop("which is contains values greater than the maximum number of documents.")
+    
+    # get proper alpha
+    alpha <- format_alpha(
+      x$alpha, 
+      k = nrow(x$beta)
+    )
+    
+    # extract dirichlet parameters for theta
+    mat <- x$counts$Cd[which, ]
+    
+    if (length(which) == 1) {
+      mat <- matrix(mat, nrow = 1)
     }
+    
+    dir_par <- t(mat) + alpha$alpha
+    
+    colnames(dir_par) <- rownames(x$theta)[which]
+    rownames(dir_par) <- colnames(x$theta)
+    
+    
+    
   } else {
-    obj <- x$beta_par
-    # do one more check on which
-    if (any(which > ncol(obj))) {
-      stop("which is contains values greater than the maximum number of topics.")
+    # get proper eta
+    eta <- format_eta(
+      x$eta, 
+      k = nrow(x$beta), 
+      Nv = ncol(x$beta)
+    )
+    
+    # extract dirichlet parameters for beta
+    dir_par <- x$counts$Cv[which, ] + eta$eta[which, ]
+    
+    if (length(which) == 1) {
+      dir_par <- matrix(dir_par, nrow = 1)
     }
+    
+    
+    rownames(dir_par) <- rownames(x$beta)[which]
+    colnames(dir_par) <- colnames(x$beta)
+    
+    dir_par <- t(dir_par)
   }
   
-  # reformat for efficient sampling
-  result <- as.data.frame(obj[, which], stringsAsFactors = FALSE)
-  
   # sample
+  result <- generate_sample(
+    dir_par = dir_par,
+    matrix = matrix,
+    times = times
+  )
+  
+  result
+}
+
+#' Generate a sample of LDA posteriors
+#' @keywords internal
+#' @description
+#'   Helper function called by both posterior.tidylda and predict.tidylda to
+#'   generate samples from the posterior.
+#' @param dir_par matrix of Dirichlet hyperparameters, one column per
+#' @param matrix character of "theta" or "beta", indicating which posterior
+#'   matrix \code{dir_par}'s columns are from.
+#' @param times Integer, number of samples to draw.
+#' @return Returns a tibble with one row per parameter per sample.
+generate_sample <- function(
+  dir_par,
+  matrix,
+  times
+) {
+  
   result <- lapply(
-    result,
-    function(y) {
+    X = as.data.frame(dir_par),
+    FUN = function(y) {
       samp <- gtools::rdirichlet(n = times, alpha = y)
       
       samp <- as.data.frame(samp, stringsAsFactors = FALSE)
       
-      colnames(samp) <- rownames(obj)
-
+      colnames(samp) <- rownames(dir_par)
+      
       out <- as.data.frame(t(samp))
       
       colnames(out) <- 1:ncol(out)
       
-      out$idx1 <- rownames(obj)
+      out$idx1 <- rownames(dir_par)
       
       out <- tidyr::pivot_longer(
         out,
-        -.data$idx1,
+        -idx1,
         names_to = "sample"
       )
       
@@ -192,13 +183,14 @@ generate.tidylda_posterior <- function(
     }
   )
   
-  # reformat the output so it's tidy
+  
+  # prepare and return result so it's tidy
   for (j in seq_along(result)) {
-    result[[j]]$idx2 <- colnames(obj)[which[j]]
+    result[[j]]$idx2 <- colnames(dir_par)[j]
   }
   
   result <- do.call(rbind, result)
-
+  
   result <- tibble::as_tibble(result)
   
   result$sample <- as.numeric(result$sample)
@@ -223,4 +215,4 @@ generate.tidylda_posterior <- function(
   }
   
   result
-}
\ No newline at end of file
+}
